/*
 * say_gr.c
 * 12-16-2004 : Support for Greek added by InAccess Networks (work funded by HOL, www.hol.gr)
 *  						George Konstantoulakis <gkon@inaccessnetworks.com>
 */

/*********** START GREEK ********************/

/*
 *  	A list of the files that you need to create
 ->  	digits/xilia = "xilia"
 ->  	digits/myrio = "ekatomyrio"
 ->  	digits/thousands = "xiliades"
 ->  	digits/millions = "ektatomyria"
 ->  	digits/[1..12]   :: A pronunciation of th digits form 1 to 12 e.g. "tria"
 ->  	digits/[10..100]  :: A pronunciation of the tens from 10 to 90 
															 e,g 80 = "ogdonta" 
						 Here we must note that we use digits/tens/100 to utter "ekato"
						 and digits/hundred-100 to utter "ekaton"
 ->  	digits/hundred-[100...1000] :: A pronunciation of  hundreds from 100 to 1000 e.g 400 = 
																		 "terakosia". Here again we use hundreds/1000 for "xilia" 
						 and digits/thousnds for "xiliades"
*/

static int say_number_full_gr(say_args_t *a, int num, const char *options)
{
	int res = 0;
	char fn[256] = "";
	int i=0;

	if (!num) { /* XXX maybe use generic code ? */
		snprintf(fn, sizeof(fn), "digits/0");
		return s_streamwait2(a, fn);
	}

	while(!res && num ) {
		i++;
		if (num < 13) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num <= 100) {
			/* 13 < num <= 100  */
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10); 
		} else if (num < 200) {
			/* 100 < num < 200 */
			snprintf(fn, sizeof(fn), "digits/hundred-100");
			num -= ((num / 100) * 100);
		}else if (num < ONE_THOUSAND) {
			/* 200 < num < 1000 */
			snprintf(fn, sizeof(fn), "digits/hundred-%d", (num/100)*100);
			num -= ((num / 100) * 100);
		}else if (num < 2000){
			snprintf(fn, sizeof(fn), "digits/xilia");
			num -= ((num / ONE_THOUSAND) * ONE_THOUSAND);
		}
		else {
			/* num >  1000 */ 
			if (num < ONE_MILLION) {
				res = say_number_full_gr(a, num / ONE_THOUSAND, options);
				if (res)
					return res;
				num = num % ONE_THOUSAND;
				snprintf(fn, sizeof(fn), "digits/thousands");
			}  else {
				if (num < ONE_BILLION) { /* 1,000,000,000 */
					res = say_number_full_gr(a, num / ONE_MILLION, options);
					if (res)
						return res;
					num = num % ONE_MILLION;
					snprintf(fn, sizeof(fn), "digits/millions");
				} else {
					ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
					res = -1;
				}
			}
		} 
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}


/*
 * digits/female-[1..4] : "Mia, dyo , treis, tessereis"
 */
static int gr_say_number_female(int num, struct ast_channel *chan, const char *ints, const char *lang){
	int tmp;
	int left;
	int res;
	char fn[256] = "";

	/* ast_log(LOG_DEBUG, "\n\n Saying number female %s %d \n\n",lang, num); */
	if (num < 5) {
		snprintf(fn, sizeof(fn), "digits/female-%d", num);
		res = wait_file(chan, ints, fn, lang);
	} else if (num < 13) {
		res = ast_say_number(chan, num, ints, lang, (char *) NULL);
	} else if (num <100 ) { 
		tmp = (num/10) * 10;
		left = num - tmp;
		snprintf(fn, sizeof(fn), "digits/%d", tmp);
		res = streamwait(chan, fn, lang, ints);
		if (left)
			gr_say_number_female(left, chan, ints, lang);
			
	} else {
		return -1;
	}
	return res;
}

/*
 * The format is  weekday - day - month -year
 * 
 * A list of the files that you need to create
 * digits/day-[1..7]  : "Deytera .. Paraskeyh"
 * digits/months/1..12 : "Ianouariou .. Dekembriou"  
													Attention the months are in 
				"gekinh klhsh"
 */


static int say_date_gr(struct ast_channel *chan, struct tm *tm1, const char *ints, const char *lang)
{
	struct tm tm = *tm1;
	char fn[256];
	int res = 0;
	
	/* W E E K - D A Y */
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/day-%d", tm.tm_wday);
		res = streamwait(chan, fn, lang, ints);
	}
	/* D A Y */
	if (!res) {
		gr_say_number_female(tm.tm_mday, chan, ints, lang);
	}
	/* M O N T H */
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
		res = streamwait(chan, fn, lang, ints);
	}
	/* Y E A R */
	if (!res)
		res = ast_say_number(chan, tm.tm_year + 1900, ints, lang, (char *) NULL);
	return res; 
}


 
/* A list of the files that you need to create
 * digits/female/1..4 : "Mia, dyo , treis, tesseris "
 * digits/kai : "KAI"
 * didgits : "h wra"
 * digits/p-m : "meta meshmbrias" 
 * digits/a-m : "pro meshmbrias"
 */

static int say_time_gr(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
{

	struct tm tm;
	int res = 0;

	localtime_r(&t,&tm);
 
	res = gr_say_number_female(hr_to_12(tm.tm_hour), chan, ints, lang);
	if (tm.tm_min) {
		if (!res)
			res = streamwait(chan, "digits/kai", lang, ints);
		if (!res)
			res = ast_say_number(chan, tm.tm_min, ints, lang, (char *) NULL);
	} else {
		if (!res)
			res = streamwait(chan, "digits/hwra", lang, ints);
	}
	if (!res)
		res = streamwait(chan, hr_to_am_pm(tm.tm_hour), lang, ints);
	return res;
}


static int say_datetime_gr(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
{
	struct tm tm;
	char fn[256];
	int res = 0;
	localtime_r(&t,&tm);

	/* W E E K - D A Y */
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/day-%d", tm.tm_wday);
		res = streamwait(chan, fn, lang, ints);
	}
	/* D A Y */
	if (!res)
		gr_say_number_female(tm.tm_mday, chan, ints, lang);
	/* M O N T H */
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
		res = streamwait(chan, fn, lang, ints);
	}
	if (!res)
		res = say_time_gr(chan, t, ints, lang);
	return res;
}

static int say_date_with_format_gr(struct ast_channel *chan, struct tm *tm1, const char *ints, const char *lang, const char *format, time_t delta)
{
	
	struct tm tm;
	int res=0 /* becomes non-zero on DTMF */, offset;
	char c, nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_gr; /* shorthand for recursive call */
	
	say_args_t a = { chan, ints, lang, -1, -1 };
	for (offset=0; res==0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
		case '\'': /* Literal name of a sound file */
			res = play_literal(chan, ints, lang, format, &offset);
			break;
		case 'A':
		case 'a':
			/* Sunday - Saturday */
			snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
			res = wait_file(chan,ints,nextmsg,lang);
			break;
		case 'B':
		case 'b':
		case 'h':
			/* January - December */
			snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
			res = wait_file(chan,ints,nextmsg,lang);
			break;
		case 'd':
		case 'e':
			/* first - thirtyfirst */
			gr_say_number_female(tm.tm_mday, chan, ints, lang);
			break;
		case 'Y':
			/* Year */
			say_number_full_gr(&a, 1900+tm.tm_year, NULL);
			break;
		case 'I':
		case 'l':
			/* 12-Hour */
			gr_say_number_female(hr_to_12(tm.tm_hour), chan, ints, lang);
			break;
		case 'H':
		case 'k':
			/* 24-Hour */
			gr_say_number_female(tm.tm_hour, chan, ints, lang);
			break;
		case 'M':
			/* Minute */
			if (tm.tm_min) {
				if (!res)
					res = streamwait(chan, "digits/kai", lang, ints);
				if (!res)
					res = say_number_full_gr(&a, tm.tm_min, NULL);
			} else {
				if (!res)
					res = streamwait(chan, "digits/oclock", lang, ints);
			}
			break;
		case 'P':
		case 'p':
			/* AM/PM */
			res = wait_file(chan,ints, hr_to_am_pm(tm.tm_hour), lang);
			break;
		case 'Q': /* Shorthand for "Today", "Yesterday", or ABdY */
		case 'q': /* Shorthand for "" (today), "Yesterday", A (weekday), or ABdY */
			if (delta >= 0) { /* Today */
				if (c == 'Q')
					res = wait_file(chan,ints, "digits/today",lang);
			} else if (delta >= -ONE_DAY) { /* Yesterday */
				res = wait_file(chan,ints, "digits/yesterday",lang);
			} else if (c == 'q' && delta >= - SIX_DAYS) {
				/* Within the last week */
				res = f(chan, &tm, ints, lang, "A", delta);
			} else {
				res = f(chan, &tm, ints, lang, "AdBY", delta);
			}
			break;
		case 'R':
			res = f(chan, &tm, ints, lang, "HM", delta);
			break;
		case 'S':
			/* Seconds */
			res = wait_file(chan,ints, "digits/kai", lang);
			if (!res)
				res = say_number_full_gr(&a, tm.tm_sec, NULL);
			if (!res)
				res = wait_file(chan,ints, "digits/seconds",lang);
			break;
		case 'T':
			res = f(chan, &tm, ints, lang, "HMS", delta);
			break;
		case ' ':
		case '\t':
			/* Just ignore spaces and tabs */
			break;
		default:
			/* Unknown character */
			ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
	}
	return res;
}

static struct _lang_sw map_gr = {
	"gr",
	0,
	F(number_full_gr),
	MISSING(enumeration_full_gr),
	F(date_gr),
	F(date_with_format_gr),
	F(time_gr),
	F(datetime_gr),
	MISSING(datetime_from_now_gr),
};

