/*
 * say_it.c
 */

/*! \brief  say_number_full_it:  Italian */
static int say_number_full_it(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";

	if (!num)
		return say_digits_full(a, 0);

	/*
	Italian support

	Like english, numbers up to 20 are a single 'word', and others
	compound, but with exceptions.
	For example 21 is not twenty-one, but there is a single word in 'it'.
	Idem for 28 (ie when a the 2nd part of a compund number
	starts with a vowel)

	There are exceptions also for hundred, thousand and million.
	In english 100 = one hundred, 200 is two hundred.
	In italian 100 = cento , like to say hundred (without one),
	200 and more are like english.
	
	Same applies for thousand:
	1000 is one thousand in en, 2000 is two thousand.
	In it we have 1000 = mille , 2000 = 2 mila 

	For million(s) we use the plural, if more than one
	Also, one million is abbreviated in it, like on-million,
	or 'un milione', not 'uno milione'.
	So the right file is provided.
	*/

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else if (num < 20) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else if (num < ONE_HUNDRED) {
			int units = num % 10;
			if (units == 1 || units == 8) { /* single word */
				snprintf(fn, sizeof(fn), "digits/%d", num);
				num = 0;
			} else {	/* two words */
				snprintf(fn, sizeof(fn), "digits/%d", num - units);
				num = units;
			}
		} else if (num < ONE_THOUSAND) {
			int hundreds = num / ONE_HUNDRED;
			if (hundreds == 1)
				snprintf(fn, sizeof(fn), "digits/hundred");
			else {
				snprintf(fn, sizeof(fn), "digits/%d", hundreds);
				playh++;
			}
			num %= ONE_HUNDRED;
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			int thousands = num/ONE_THOUSAND;
			if (thousands == 1)
				snprintf(fn, sizeof(fn), "digits/thousand");
			else {
				res = say_number_full_it(a, thousands, options);
				if (res)
					break;
				snprintf(fn, sizeof(fn), "digits/thousands");
			}
			num = num % ONE_THOUSAND;
		} else if (num < ONE_BILLION) { /* 1,000,000,000 */
			int millions = num / ONE_MILLION;
			if (millions == 1)
				snprintf(fn, sizeof(fn), "digits/million");
			else {
				res = say_number_full_it(a, millions, options);
				if (res)
					break;
				snprintf(fn, sizeof(fn), "digits/millions");
			}
			num = num % ONE_MILLION;
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			break;
		}
		res = s_streamwait2(a, fn);
	}
	return res;
}

static int say_date_with_format_it(struct ast_channel *chan, struct tm *tm1, const char *ints, const char *lang, const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_it; /* shorthand for recursive call */
	char c;

	for (offset=0 ; res == 0 && (c = format[offset]) != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (c) {
			/* NOTE:  if you add more options here, please try to be consistent with strftime(3) */
			case '\'': /* Literal name of a sound file */
				res = play_literal(chan, ints, lang, format, &offset);
				break;
			case 'A':
			case 'a':
				/* Sunday - Saturday */
				snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'B':
			case 'b':
			case 'h':
				/* January - December */
				snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'm':
				/* First - Twelfth */
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'd':
			case 'e':
				/* First day of the month is spelled as ordinal */
				if (tm.tm_mday == 1) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					if (!res) {
						res = ast_say_number(chan, tm.tm_mday, ints, lang, (char *) NULL);
					}
				}
				break;
			case 'Y':
				/* Year */
				if (tm.tm_year > 99) {
					res = wait_file(chan,ints, "digits/ore-2000",lang);
					if (tm.tm_year > 100) {
						if (!res) {
						/* This works until the end of 2021 */
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year - 100);
						res = wait_file(chan,ints,nextmsg,lang);
						}
					}
				} else {
					if (tm.tm_year < 1) {
						/* I'm not going to handle 1900 and prior */
						/* We'll just be silent on the year, instead of bombing out. */
					} else {
						res = wait_file(chan,ints, "digits/ore-1900",lang);
						if ((!res) && (tm.tm_year != 0)) {
							if (tm.tm_year <= 21) {
								/* 1910 - 1921 */
								snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year);
								res = wait_file(chan,ints,nextmsg,lang);
							} else {
								/* 1922 - 1999, but sounds badly in 1928, 1931, 1938, etc... */
								int ten, one;
								ten = tm.tm_year / 10;
								one = tm.tm_year % 10;
								snprintf(nextmsg,sizeof(nextmsg), "digits/%d", ten * 10);
								res = wait_file(chan,ints,nextmsg,lang);
								if (!res) {
									if (one != 0) {
										snprintf(nextmsg,sizeof(nextmsg), "digits/%d", one);
										res = wait_file(chan,ints,nextmsg,lang);
									}
								}
							}
						}
					}
				}
				break;
			case 'I':
			case 'l':
				/* 12-Hour */
				snprintf(nextmsg,sizeof(nextmsg), "digits/%d", hr_to_12(tm.tm_hour));
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'H':
			case 'k':
				/* 24-Hour */
				if (tm.tm_hour == 0) {
					res = wait_file(chan,ints, "digits/ore-mezzanotte",lang);
				} else if (tm.tm_hour == 1) {
					res = wait_file(chan,ints, "digits/ore-una",lang);
				} else {
					res = ast_say_number(chan, tm.tm_hour, ints, lang, (char *) NULL);
				}
				break;
			case 'M':
				/* Minute */
				res = ast_say_number(chan, tm.tm_min, ints, lang, (char *) NULL);
				break;
			case 'P':
			case 'p':
				/* AM/PM */
				res = wait_file(chan,ints, hr_to_am_pm(tm.tm_hour), lang);
				break;
			case 'Q': /* Shorthand for "Today", "Yesterday", or ABdY */
			case 'q': /* Shorthand for "" (today), "Yesterday", A (weekday), or ABdY */
					if (delta >= 0) { /* Today */
						if (c == 'Q')
							res = wait_file(chan,ints, "digits/today",lang);
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = wait_file(chan,ints, "digits/yesterday",lang);
					} else if (c == 'q' && delta >= - SIX_DAYS) {
						/* Within the last week */
						res = f(chan, &tm, ints, lang, "A", delta);
					} else {
						res = f(chan, &tm, ints, lang, "AdB", delta);
					}
				break;
			case 'R':
				res = f(chan, &tm, ints, lang, "HM", delta);
	        	break;
			case 'S':
				/* Seconds */
				if (tm.tm_sec == 0) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = wait_file(chan,ints,nextmsg,lang);
				} else if (tm.tm_sec < 10) {
					res = wait_file(chan,ints, "digits/oh",lang);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
						res = wait_file(chan,ints,nextmsg,lang);
					}
				} else if ((tm.tm_sec < 21) || (tm.tm_sec % 10 == 0)) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					int ten, one;
					ten = (tm.tm_sec / 10) * 10;
					one = (tm.tm_sec % 10);
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", ten);
					res = wait_file(chan,ints,nextmsg,lang);
					if (!res) {
						/* Fifty, not fifty-zero */
						if (one != 0) {
							snprintf(nextmsg,sizeof(nextmsg), "digits/%d", one);
							res = wait_file(chan,ints,nextmsg,lang);
						}
					}
				}
		        break;
			case 'T':
				res = f(chan, &tm, ints, lang, "HMS", delta);
				break;
			case ' ':
			case '\t':
				/* Just ignore spaces and tabs */
				break;
			default:
				/* Unknown character */
				ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
	}
	return res;
}

static struct _lang_sw map_it = {
	"it",
	0,
	F(number_full_it),
	MISSING(enumeration_full_it),
	MISSING(date_it),
	F(date_with_format_it),
	MISSING(time_it),
	MISSING(datetime_it),
	MISSING(datetime_from_now_it),
};

