/*
 * say_tw.c
 */

/*! \brief  say_number_full_tw: Taiwanese syntax */
static int say_number_full_tw(say_args_t *a, int num, const char *options)
{
	int res = 0;
	int playh = 0;
	char fn[256] = "";
	if (!num)
		return say_digits_full(a, 0);

	while(!res && (num || playh)) {
		if (num < 0) {
			snprintf(fn, sizeof(fn), "digits/minus");
			num = negate(num);
		} else if (playh) {
			snprintf(fn, sizeof(fn), "digits/hundred");
			playh = 0;
		} else	if (num < 10) {
			snprintf(fn, sizeof(fn), "digits/%d", num);
			num = 0;
		} else	if (num < 100) {
			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
			num -= ((num / 10) * 10);
		} else if (num < ONE_THOUSAND){
			snprintf(fn, sizeof(fn), "digits/%d", (num/100));
			playh++;
			num -= ((num / 100) * 100);
		} else if (num < ONE_MILLION) { /* 1,000,000 */
			res = say_number_full_tw(a, num / ONE_THOUSAND, options);
			if (res)
				return res;
			num = num % ONE_THOUSAND;
			snprintf(fn, sizeof(fn), "digits/thousand");
		} else if (num < ONE_BILLION) {	/* 1,000,000,000 */
			res = say_number_full_tw(a, num / ONE_MILLION, options);
			if (res)
				return res;
			num = num % ONE_MILLION;
			snprintf(fn, sizeof(fn), "digits/million");
		} else {
			ast_log(LOG_DEBUG, "Number '%d' is too big for me\n", num);
			res = -1;
		}
		if (!res)
			res = s_streamwait3(a, fn);
	}
	return res;
}

/* Taiwanese syntax */
static int say_date_with_format_tw(struct ast_channel *chan,
	struct tm *tm1, const char *ints, const char *lang, const char *format, time_t delta)
{
	struct tm tm = *tm1;
	int res=0, offset;
	char nextmsg[256];
	say_date_with_format_f *f = say_date_with_format_tw; /* shorthand for recursive call */

	for (offset=0 ; format[offset] != '\0' ; offset++) {
		ast_log(LOG_DEBUG, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
		switch (format[offset]) {
			/* NOTE:  if you add more options here, please try to be consistent with strftime(3) */
			case '\'': /* Literal name of a sound file */
				res = play_literal(chan, ints, lang, format, &offset);
				break;
			case 'A':
			case 'a':
				/* Sunday - Saturday */
				snprintf(nextmsg,sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'B':
			case 'b':
			case 'h':
				/* January - December */
				snprintf(nextmsg,sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'm':
				/* First - Twelfth */
				snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mon +1);
				res = wait_file(chan,ints,nextmsg,lang);
				break;
			case 'd':
			case 'e':
				/* First - Thirtyfirst */
				if (!(tm.tm_mday % 10) || (tm.tm_mday < 10)) {
					snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/h-%dh", tm.tm_mday - (tm.tm_mday % 10));
					res = wait_file(chan,ints,nextmsg,lang);
					if(!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/h-%d", tm.tm_mday % 10);
						res = wait_file(chan,ints,nextmsg,lang);
					}
				}
				break;
			case 'Y':
				/* Year */
				if (tm.tm_year > 99) {
					res = wait_file(chan,ints, "digits/2",lang);
					if (!res) {
						res = wait_file(chan,ints, "digits/thousand",lang);
					}
					if (tm.tm_year > 100) {
						if (!res) {
							snprintf(nextmsg,sizeof(nextmsg), "digits/%d", (tm.tm_year - 100) / 10);
							res = wait_file(chan,ints,nextmsg,lang);
							if (!res) {
								snprintf(nextmsg,sizeof(nextmsg), "digits/%d", (tm.tm_year - 100) % 10);
								res = wait_file(chan,ints,nextmsg,lang);
							}
						}
					}
					if (!res) {
						res = wait_file(chan,ints, "digits/year",lang);
					}
				} else {
					if (tm.tm_year < 1) {
						/* I'm not going to handle 1900 and prior */
						/* We'll just be silent on the year, instead of bombing out. */
					} else {
						res = wait_file(chan,ints, "digits/1",lang);
						if (!res) {
							res = wait_file(chan,ints, "digits/9",lang);
						}
						if (!res) {
							if (tm.tm_year <= 9) {
								/* 1901 - 1909 */
								res = wait_file(chan,ints, "digits/0",lang);
								if (!res) {
									snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year);
									res = wait_file(chan,ints,nextmsg,lang);
								}
							} else {
								/* 1910 - 1999 */
								snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year / 10);
								res = wait_file(chan,ints,nextmsg,lang);
								if (!res) {
									snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_year % 10);
									res = wait_file(chan,ints,nextmsg,lang);
								}
							}
						}
					}
					if (!res) {
						res = wait_file(chan,ints, "digits/year",lang);
					}
				}
				break;
			case 'I':
			case 'l':
				/* 12-Hour */
				snprintf(nextmsg,sizeof(nextmsg), "digits/%d", hr_to_12(tm.tm_hour));
				res = wait_file(chan,ints,nextmsg,lang);
				if (!res) {
					res = wait_file(chan,ints, "digits/oclock",lang);
				}
				break;
			case 'H':
			case 'k':
				/* 24-Hour */
				if (!(tm.tm_hour % 10) || tm.tm_hour < 10) {
					if (tm.tm_hour < 10) {
						res = wait_file(chan, ints, "digits/0", lang);
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour - (tm.tm_hour % 10));
					res = wait_file(chan,ints,nextmsg,lang);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_hour % 10);
						res = wait_file(chan,ints,nextmsg,lang);
					}
				}
				if (!res) {
					res = wait_file(chan,ints, "digits/oclock",lang);
				}
				break;
			case 'M':
				/* Minute */
				if (!(tm.tm_min % 10) || tm.tm_min < 10) {
					if (tm.tm_min < 10) {
						res = wait_file(chan, ints, "digits/0", lang);
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min - (tm.tm_min % 10));
					res = wait_file(chan,ints,nextmsg,lang);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_min % 10);
						res = wait_file(chan,ints,nextmsg,lang);
					}
				}
				if (!res) {
					res = wait_file(chan,ints, "digits/minute",lang);
				}
				break;
			case 'P':
			case 'p':
				/* AM/PM */
				res = wait_file(chan,ints, hr_to_am_pm(tm.tm_hour), lang);
				break;
			case 'Q':
				/* Shorthand for "Today", "Yesterday", or ABdY */
					if (delta >= 0) { /* Today */
						res = wait_file(chan,ints, "digits/today",lang);
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = wait_file(chan,ints, "digits/yesterday",lang);
					} else {
						res = f(chan, &tm, ints, lang, "YBdA", delta);
					}
				break;
			case 'q':
				/* Shorthand for "" (today), "Yesterday", A (weekday), or ABdY */
					if (delta >= 0) { /* Today */
					} else if (delta >= -ONE_DAY) { /* Yesterday */
						res = wait_file(chan,ints, "digits/yesterday",lang);
					} else if (delta >= - SIX_DAYS) {
						/* Within the last week */
						res = f(chan, &tm, ints, lang, "A", delta);
					} else {
						res = f(chan, &tm, ints, lang, "YBdA", delta);
					}
				break;
			case 'R':
				res = f(chan, &tm, ints, lang, "HM", delta);
				break;
			case 'S':
				/* Seconds */
				if (!(tm.tm_sec % 10) || tm.tm_sec < 10) {
					if (tm.tm_sec < 10) {
						res = wait_file(chan, ints, "digits/0", lang);
					}
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec);
					res = wait_file(chan,ints,nextmsg,lang);
				} else {
					snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec - (tm.tm_sec % 10));
					res = wait_file(chan,ints,nextmsg,lang);
					if (!res) {
						snprintf(nextmsg,sizeof(nextmsg), "digits/%d", tm.tm_sec % 10);
						res = wait_file(chan,ints,nextmsg,lang);
					}
				}
				if (!res) {
					res = wait_file(chan,ints, "digits/second",lang);
				}
				break;
			case 'T':
				res = f(chan, &tm, ints, lang, "HMS", delta);
				break;
			case ' ':
			case '	':
				/* Just ignore spaces and tabs */
			break;
			default:
				/* Unknown character */
				ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
		}
		/* Jump out on DTMF */
		if (res) {
			break;
		}
	}
	return res;
}

/* Taiwanese syntax */
static int say_time_tw(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
{
	struct tm tm;
	int res = 0;

	localtime_r(&t,&tm);
	if (!res)
		res = streamwait(chan, hr_to_am_pm(tm.tm_hour), lang, ints);
	if (!res)
		res = ast_say_number(chan, hr_to_12(tm.tm_hour), ints, lang, NULL);
	if (!res)
		res = streamwait(chan, "digits/oclock", lang, ints);
	if (!res)
		res = ast_say_number(chan, tm.tm_min, ints, lang, (char *) NULL);
	if (!res)
		res = streamwait(chan, "digits/minute", lang, ints);
	return res;
}

/* Taiwanese syntax */
static int say_datetime_tw(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
{
	struct tm tm;
	char fn[256];
	int res = 0;
	localtime_r(&t,&tm);
	if (!res)
		res = ast_say_number(chan, tm.tm_year + 1900, ints, lang, (char *) NULL);
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
		res = ast_streamfile(chan, fn, lang);
		if (!res)
			res = ast_waitstream(chan, ints);
	}
	if (!res)
		res = ast_say_number(chan, tm.tm_mday, ints, lang, (char *) NULL);
	if (!res) {
		snprintf(fn, sizeof(fn), "digits/day-%d", tm.tm_wday);
		res = ast_streamfile(chan, fn, lang);
 		if (!res)
			res = ast_waitstream(chan, ints);
	}
	if (!res)
		res = streamwait(chan, hr_to_am_pm(tm.tm_hour), lang, ints);
	if (!res)
		res = ast_say_number(chan, hr_to_12(tm.tm_hour), ints, lang, NULL);
	if (!res)
		res = streamwait(chan, "digits/oclock", lang, ints);
	if (!res)
		res = ast_say_number(chan, tm.tm_min, ints, lang, (char *) NULL);
	if (!res)
		res = streamwait(chan, "digits/minute", lang, ints);
	return res;
}

static struct _lang_sw map_tw = {
	"tw",
	0,
	F(number_full_tw),
	MISSING(enumeration_full_tw),
	MISSING(date_tw),
	F(date_with_format_tw),
	F(time_tw),
	F(datetime_tw),
	MISSING(datetime_from_now_tw),
};

